$VERBOSE = false

######################################################################
# Setup logging

require "logger"
log = Logger.new(STDERR)

######################################################################
# Setup Agent

require "jabber-rpc/rpc"

class TodoAgent < Jabber::RPC::BasicAgent
  COMMAND_REGEX = /^\s*(\w+)\s*[-:]?\s*/i

  def handle_message(msg)
    if msg.body =~ COMMAND_REGEX
      method = Regexp.last_match[1].downcase
      body = Regexp.last_match.post_match
      body = nil if body.empty?
      if exposed_commands.include?(method)
        result = nil
        if body
          args = send(method + "_human_in", body)
          result = send(method, msg.from, *args)
        else
          result = send(method, msg.from)
        end
        result = send(method + "_human_out", result)
        msg.reply.set_body(result).send
      else 
        msg.reply.set_body("uhm?").send
      end
    else
      msg.reply.set_body("uhm?").send 
    end      
  end

  def exposed_commands
    ["list", "todo"]
  end

  def todo_human_in(body)
    [body]
  end

  def todo_human_out(result)
    case result
    when true then "Todo item recorded"
    when false then "Error occurred"
    end
  end

  def todo(from, body)
    TodoItem.create("author" => "%s@%s" % [from.node, from.host], "body" => body)
    true
  end

  def list_human_out(result)  
    result.join("\n")
  end

  def list(from)
    TodoItem.
      find_all([ "author = '%s@%s'", from.node, from.host ]).
      collect {|item| "id %d: %s" % [item.id, item.body]}
  end
end


######################################################################
# Handling command line options

require "optparse"
require "ostruct"

options = OpenStruct.new
options.jid = "bot@localhost/TodoAgent"
options.password = "bot"
options.db = "test"
options.dbuser = "root"
options.dbpassword = ""
options.dbadapter = "mysql"
options.dbdatabase = "test"

opts = OptionParser.new do |opts|
  opts.banner = "Usage: todo_agent.rb [options]"
  opts.separator ""
  opts.on("-u", "--user STRING", "--jid STRING", "Jabber ID to bind to the server with (default: bot@localhost/Todo)") {|options.jid|}
  opts.on("-p", "--password STRING", "Password to bind to the server with (default: bot)") {|options.password|}
  opts.on("--db-username STRING", "User to access the database (default: root)") {|options.dbusername|}
  opts.on("--db-password STRING", "Password to access the database (default: empty)") {|options.dbpassword|}
  opts.on("--db-host STRING", "Host where the database resides (default: localhost)") {|options.dbhost|}
  opts.on("--db-adapter STRING", "Database type as recognized by ActiveRecord (default: mysql)") {|options.dbadapter|}
  opts.on("--db-database STRING", "Database name (default: test)") {|options.dbdatabase|}
  opts.on_tail("-h", "--help", "Show this message") do
    puts opts
    exit
  end
  opts.separator ""
  opts.parse!
end

######################################################################
# Setup database

require "active_record"

log.info "Establishing database connection"
ActiveRecord::Base.establish_connection(
  :adapter => options.dbadapter,
  :host => options.dbhost,
  :username => options.dbusername,
  :password => options.dbpassword,
  :database => options.dbdatabase
  )

class TodoItem < ActiveRecord::Base  
end

log.info "Preparing database tables"
ActiveRecord::Base.connection.execute "drop table if exists todo_items"
ActiveRecord::Base.connection.execute "create table todo_items (id int(11) auto_increment, author varchar(100), body text, primary key (id))"

######################################################################
# Run agent

begin
  current_thread = Thread.current
  Signal.trap("HUP") { current_thread.wakeup }

  log.info "Binding agent to server"
  agent = TodoAgent.connect(options.jid, options.password)
  log.info "Agent running"

  log.info <<END

You can now talk to #{options.jid} over Jabber.  

Start an IM program and issue the command `todo blah blah blah' (to
record a todo item) or the command `list' (to show recorded items).

Alternatively, from Ruby with jabber-rpc module:

  todo_agent = Jabber::RPC::Client.new(session, #{options.jid}).proxy3
  todo_agent.todo("do this and that")
  puts todo_agent.list

Each user requesting the service gets his own todo list.  Enjoy!  To
quit, press Ctrl+C.
END

  Thread.stop
rescue Exception => e
  log.error e
ensure
  agent.disconnect if agent
end

